#include "mex.h"
#include "matrix.h"
#include "stdio.h"
#include "stdlib.h"
#include "assert.h"

void nzz(const bool * matrix, const size_t n, int* n_nz, int** nz)
{
	assert(matrix);
	int m = 0;
	for (int i = 0; i < n; i++) {
		if (matrix[i]) {
			m++;
		}
	}
	*n_nz = m;
	//printf("value m: %d\n", m);


	int *nz_temp = malloc(m * sizeof(int));

	int j = 0;
	for (int i = 0; i < n; i++) {
		if (matrix[i]) {
			nz_temp[j] = i;
			j++;
		}
	}
	*nz = nz_temp;

	return;
}


void matrixComponentSearch(const bool *matrix, const size_t n, const int row_size, double *out_matrix)
/*************************************************************************
matrixComponentSearch - Identify components in matrix
*************************************************************************/
{
	int current_component = 0;

	int n_nz;
	int *indices;
	int neighbor;

	bool* visited = calloc(n, sizeof(bool));
	int* queue = malloc(n * sizeof(int));

	nzz(matrix, n, &n_nz, &indices);
	
	//printf("n_nz: %d\n", n_nz);
	
	for (int current_index = 0; current_index < n_nz; current_index++) {
		if (visited[indices[current_index]]) {
			continue;
		}
		//printf("Processing: %d\n", indices[current_index]);

		current_component++;

		int q_index = 0;
		queue[q_index] = indices[current_index];
		visited[queue[q_index]] = 1;
		out_matrix[queue[q_index]] = current_component;

		//printf("Assigning %d to %d\n", queue[q_index], current_component);

		while (q_index >= 0) {
			//printf("Dequeuing %d\n", queue[q_index]);
			int current_node = queue[q_index];
			q_index--;

			/* check neighbor above */
			if (current_node % row_size != 0) {
				neighbor = current_node - 1;
				//printf("Checking %d with val %.2f and visited %d\n", neighbor, matrix[neighbor], visited[neighbor]);
				if (matrix[neighbor] && !visited[neighbor]) {
					visited[neighbor] = 1;
					//printf("Assigning %d to %d\n", neighbor, current_component);
					out_matrix[neighbor] = current_component;
					q_index++;
					//printf("Queuing %d\n", neighbor);
					queue[q_index] = neighbor;
				}
			}
			/* check neighbor below */
			if (current_node % row_size != row_size - 1) {
				neighbor = current_node + 1;
				//printf("Checking %d with val %.2f and visited %d\n", neighbor, matrix[neighbor], visited[neighbor]);
				if (matrix[neighbor] && !visited[neighbor]) {
					visited[neighbor] = 1;
					//printf("Assigning %d to %d\n", neighbor, current_component);
					out_matrix[neighbor] = current_component;
					q_index++;
					//printf("Queuing %d\n", neighbor);
					queue[q_index] = neighbor;
				}
			}
			/* check neighbor left */
			if (current_node - row_size >= 0) {
				neighbor = current_node - row_size;
				//printf("Checking %d with val %.2f and visited %d\n", neighbor, matrix[neighbor], visited[neighbor]);
				if (matrix[neighbor] && !visited[neighbor]) {
					visited[neighbor] = 1;
					//printf("Assigning %d to %d\n", neighbor, current_component);
					out_matrix[neighbor] = current_component;
					q_index++;
					//printf("Queuing %d\n", neighbor);
					queue[q_index] = neighbor;
				}
			}
			/* check neighbor right */
			if (current_node + row_size < n) {
				neighbor = current_node + row_size;
				//printf("Checking %d with val %.2f and visited %d\n", neighbor, matrix[neighbor], visited[neighbor]);
				if (matrix[neighbor] && !visited[neighbor]) {
					visited[neighbor] = 1;
					//printf("Assigning %d to %d\n", neighbor, current_component);
					out_matrix[neighbor] = current_component;
					q_index++;
					//printf("Queuing %d\n", neighbor);
					queue[q_index] = neighbor;
				}
			}
		}
	}

	free(queue);
	free(indices);
	free(visited);
}


void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray*prhs[])
/*************************************************************************
mexFunction - Main function
*************************************************************************/
{

  /* Declare variables */
  int n;
  bool *matrix;
  double *matrix_out;
  const size_t *dim_array;

  /* Check for proper number of input and output arguments */
  if (nrhs != 1) {
      mexErrMsgIdAndTxt( "matrixComponentSearch:invalidNumInputs",
              "One input argument required.");
  }
  if (nlhs > 1){
      mexErrMsgIdAndTxt( "matrixComponentSearch:findnz:maxlhs",
              "Too many output arguments.");
  }

  /* Check data type of input argument */
  if (!mxIsLogical(prhs[0])) {
      mexErrMsgIdAndTxt( "matrixComponentSearch:invalidInputType",
              "Input array must be logical.");
  }

  /* Check matrix is two-dimensional. */
  if (mxGetNumberOfDimensions(prhs[0]) != 2) {
    mexErrMsgIdAndTxt( "matrixComponentSearch:invalidInputSize",
      "Input array must be two-dimensional.");
  }


  /* Get the number of elements in the input argument */
  n = mxGetNumberOfElements(prhs[0]);

  /* Get the data */
  matrix = (bool *) mxGetData(prhs[0]);

  /* get dimensions */
  dim_array = mxGetDimensions(prhs[0]);

  /* Get the number of dimensions in the input argument. Allocate the space for the return argument */
  plhs[0] = mxCreateDoubleMatrix(dim_array[0], dim_array[1], mxREAL);

  matrix_out = (double *) mxGetPr(plhs[0]);

  matrixComponentSearch(matrix, n, dim_array[0], matrix_out);
  
  return;

}